/* cusr>login>src>setupenv.c
**
** Handle initialize environment for user.
**
**    date     sign     comment
**    890414   flg      initial coding
**    890810   flg      Removed creation of gvar file if it didn't exist.
**    890811   flg      Now starts shell if specified.
**    890907   flg      Changed name to SetupEnv
**
** Copyright (c), V{sterh|jdskolan Sk|vde.
*/
#include <string.h>
#include <charvar.h>
#include <primos/ndir.h>
#include <primos/meter.h>
#include <user_types.h>
#include "login.h"

SetupEnv()
{
   char * cp, * gvget(), * pUser, home[130], *gv;
   char * greet, guser[40], cmd[256], Friends[256];
   int j, newstime;


   /*
   ** We are only doing this if the user has one of these gvars..
   */
   if (    !fexists(gv="login.gv")              &&
           !fexists(gv="*>etc>login.gv")        &&
           !fexists(gv="*>login>login.gv")      &&
           !fexists(gv="*>etc>gvar.file")          )
      return 1;



   strcat(strcpy(cmd, "DEFGV "), gv);

   if (system(cmd) == -1)
   {
       puts("Unable to open your LOGIN.GV");
       return 0;
   }

#ifdef VHOJD
   gvset(".TERM",    MyTerm);
   gvset(".TERMPOS", MyPos);
   gvset(".TERMTYP", MyLtyp);
   gvset(".PRINTER", MyPrinter);
#endif

   if (pUser=gvget(".FRIENDS"))        /* Do we have any friens? */
   {

      /*
      ** Loop through his list of friends, and then see if he wants
      ** to send a greeting message to them.
      */
      strcpy(Friends, pUser);
      pUser = Friends;

      while(cp=strchr(pUser, ';'))     /* Extract one user from the list. */
      {
         *cp = '\0';

         if (! (greet=gvget(guser)))                     /* Does it exist? */
               greet = "Hello! I am sitting in @\207.";  /* Use default. */

         Greeting(pUser, greet);

         pUser = cp+1;              /* Skip to next. */
      }
   }

   /*
   ** Should we startup his own search rules?
   */
   if ((cp=gvget(".SR")) || (fexists(cp="*>etc>sr")))
      if (! StartSearchRules(cp))
            return 1;

   /*
   ** Does he use any private shell, that is other then std$cp.
   **
   */
   if ((cp=gvget(".SHELL")) || (cp=gvget(".SH$PARSER")))
      startparser(cp);

   return 1;
}

/***********************************************************************
** StartSearchRules(path)
**
** Start up this users search rules.
*/
StartSearchRules(path)
char * path;
{
   Dir * dh;
   char fpath[130];
   char * p;

   p = fpath+strlen(strcat(strcpy(fpath, path), ">"));

   if (! (dh=DirOpen(path)))
      return;

   while(DirSearch("@@.SR", 0, dh))
   {
      *p  = '\0';
      strcat(fpath, _name(dh));
      SetSr(fpath, BaseName(_name(dh), NULL), 0x8000);
   }

   DirClose(dh);
   return;
}

/***********************************************************************
** startparser
**
** Start a user specified parser.
**
*/
startparser(p)
char * p;
{
   extern char * dynt();
   char  * parser;
   short error = 0;
   fortran void cl$set_ev();

   if (!(parser = dynt(p)))
   {
      printf("ERROR! Can't find the routine \"%s\" in any library!\n", p);
      return 0;
   }

   error = 0;
   cl$set_ev(1, parser, error);
   if (errno=error)
      perror(p);

   return 1;
}


/*
** Greeting
**
** Send a greeting to specified user. We will first try to
** send him a direct message and then a defered.
**
*/
Greeting(user, fmt)
char * user, *fmt;
{
   char *p, format[80];
   int loggedin=0;

#ifdef VHOJD
   if (p=strchr(fmt, '@'))    /* AT sign. Replace it with "where he is". */
   {
      *p++ = '\0';
      strcat(strcpy(format, fmt), MyPos);
      strcat(format, p);
      *(p-1) = '@';     /* Put this back (and don't ask me why??) */
   }
   else
#endif
      strcpy(format, fmt);


   if (Message(1, user, "%s", format) == -1)
      Message(0, user, "%s", format);

   return;
}

/***********************************************************************
** dynt(name)
**
** Snap a dynt, and return a function pointer to the routine.
**
*/
char *dynt(name)
char *name;
{
   fortran void ckdyn$();

   char *dummy;           /* Dummy variable                */
   short error;           /* CKDYN$ error return variable  */
   charvar(32) pstring;

   union                /* Pointer union */
   {
      char *str;
      struct
      {
         unsigned fault   :  1;
         unsigned ring    :  2;
         unsigned rfu     :  1;
         unsigned segment : 12;
         unsigned short offset;
         unsigned short extension;
      } parts;
   } ptr;

   setcvar(pstring, name);
   pl1strupr    (pstring);          /* Turn string into uppercase only  */


   errno = error = 0;
   ckdyn$(pstring, error);  /* Check if DYNT:ing is possible                 */
   if (error)
   {
      errno = error;
      return((char *) 0);   /* Return a NULL pointer to indicate failed DYNT */
   }

   ptr.str = (char *) &pstring;  /* Set pointer to point to string      */
   ptr.parts.fault = 1;          /* Set fault bit in pointer            */
   ptr.parts.ring  = 0;          /* Set ring zero to indicate DYNT      */

   dummy = ptr.str;             /* Solve DYNT                          */

   return(ptr.str);              /* Return the address of the routine   */
}

pl1strupr(pstring)
charvar(32) pstring;
{
   int i;

   for(i=0;i<pstring.length;i++)
      pstring.text[i] = toupper(pstring.text[i]);
}
